@@ -29,7 +29,6 @@
 #include "gtkcelllayout.h"
 #include "gtkcellrenderertext.h"
 #include "gtkcombobox.h"
-#include "gtkcomboboxentry.h"
 #include "gtkfilechooserbutton.h"
 #include "gtkimage.h"
 #include "gtklabel.h"
@@ -41,7 +40,6 @@
 #include "gtkprivate.h"
 
 #include "gtkprinteroptionwidget.h"
-#include "gtkalias.h"
 
 #define GTK_PRINTER_OPTION_WIDGET_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_PRINTER_OPTION_WIDGET, GtkPrinterOptionWidgetPrivate))
@@ -73,7 +71,7 @@ enum {
 
 enum {
   PROP_0,
-  PROP_SOURCE,
+  PROP_SOURCE
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -288,11 +286,11 @@ static GtkWidget *
 combo_box_entry_new (void)
 {
   GtkWidget *combo_box;
-  combo_box = gtk_combo_box_entry_new ();
+  combo_box = g_object_new (GTK_TYPE_COMBO_BOX, "has-entry", TRUE, NULL);
 
   combo_box_set_model (combo_box);
 
-  gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (combo_box), NAME_COLUMN);
+  gtk_combo_box_set_entry_text_column (GTK_COMBO_BOX (combo_box), NAME_COLUMN);
 
   return combo_box;
 }
@@ -374,18 +372,11 @@ combo_box_get (GtkWidget *combo)
   gchar *value;
   GtkTreeIter iter;
 
-  if (GTK_IS_COMBO_BOX_ENTRY (combo))
-    {
-      value = gtk_combo_box_get_active_text(GTK_COMBO_BOX (combo));
-    }
-  else
-    {
-      model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 
-      value = NULL;
-      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
-         gtk_tree_model_get (model, &iter, VALUE_COLUMN, &value, -1);
-     }
+  value = NULL;
+  if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
+     gtk_tree_model_get (model, &iter, VALUE_COLUMN, &value, -1);
 
   return value;
 }
@@ -454,8 +445,8 @@ filesave_changed_cb (GtkWidget              *button,
 {
   GtkPrinterOptionWidgetPrivate *priv = widget->priv;
   gchar *uri, *file;
+  gchar *directory;
 
-  /* TODO: how do we support nonlocal file systems? */
   file = g_filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (priv->entry)),
 			       -1, NULL, NULL, NULL);
   if (file == NULL)
@@ -464,31 +455,38 @@ filesave_changed_cb (GtkWidget              *button,
   /* combine the value of the chooser with the value of the entry */
   g_signal_handler_block (priv->source, priv->source_changed_handler);  
 
-  if (g_path_is_absolute (file))
-    uri = g_filename_to_uri (file, NULL, NULL);
-  else
+  directory = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (priv->combo));
+
+  if ((g_uri_parse_scheme (file) == NULL) && (directory != NULL))
     {
-      gchar *path;
+      if (g_path_is_absolute (file))
+        uri = g_filename_to_uri (file, NULL, NULL);
+      else
+        {
+          gchar *path;
 
 #ifdef G_OS_UNIX
-      if (file[0] == '~' && file[1] == '/')
-        {
-          path = g_build_filename (g_get_home_dir (), file + 2, NULL);
-	}
-      else
+          if (file[0] == '~' && file[1] == '/')
+            {
+              path = g_build_filename (g_get_home_dir (), file + 2, NULL);
+            }
+          else
 #endif
-        {
-          gchar *directory;
+            {
+              path = g_build_filename (directory, file, NULL);
+            }
 
-          directory = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (priv->combo));
-          path = g_build_filename (directory, file, NULL);
+          uri = g_filename_to_uri (path, NULL, NULL);
 
-          g_free (directory);
+          g_free (path);
         }
-
-      uri = g_filename_to_uri (path, NULL, NULL);
-
-      g_free (path);
+    }
+  else
+    {
+      if (g_uri_parse_scheme (file) != NULL)
+        uri = g_strdup (file);
+      else
+        uri = g_build_path ("/", gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (priv->combo)), file, NULL);
     }
  
   if (uri)
@@ -496,6 +494,7 @@ filesave_changed_cb (GtkWidget              *button,
 
   g_free (uri);
   g_free (file);
+  g_free (directory);
 
   g_signal_handler_unblock (priv->source, priv->source_changed_handler);
   emit_changed (widget);
@@ -679,12 +678,14 @@ construct_widgets (GtkPrinterOptionWidget *widget)
   
   deconstruct_widgets (widget);
   
+  gtk_widget_set_sensitive (GTK_WIDGET (widget), TRUE);
+
   if (source == NULL)
     {
       priv->combo = combo_box_new ();
       combo_box_append (priv->combo,_("Not available"), "None");
       gtk_combo_box_set_active (GTK_COMBO_BOX (priv->combo), 0);
-      gtk_widget_set_sensitive (priv->combo, FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
       gtk_widget_show (priv->combo);
       gtk_box_pack_start (GTK_BOX (widget), priv->combo, TRUE, TRUE, 0);
     }
@@ -738,7 +739,7 @@ construct_widgets (GtkPrinterOptionWidget *widget)
 
     case GTK_PRINTER_OPTION_TYPE_ALTERNATIVE:
       group = NULL;
-      priv->box = gtk_hbox_new (FALSE, 12);
+      priv->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
       gtk_widget_show (priv->box);
       gtk_box_pack_start (GTK_BOX (widget), priv->box, TRUE, TRUE, 0);
       for (i = 0; i < source->num_choices; i++)
@@ -759,6 +760,8 @@ construct_widgets (GtkPrinterOptionWidget *widget)
 
     case GTK_PRINTER_OPTION_TYPE_STRING:
       priv->entry = gtk_entry_new ();
+      gtk_entry_set_activates_default (GTK_ENTRY (priv->entry),
+                                       gtk_printer_option_get_activates_default (source));
       gtk_widget_show (priv->entry);
       gtk_box_pack_start (GTK_BOX (widget), priv->entry, TRUE, TRUE, 0);
       g_signal_connect (priv->entry, "changed", G_CALLBACK (entry_changed_cb), widget);
@@ -780,9 +783,13 @@ construct_widgets (GtkPrinterOptionWidget *widget)
 
         /* TODO: make this a gtkfilechooserentry once we move to GTK */
         priv->entry = gtk_entry_new ();
-        priv->combo = gtk_file_chooser_button_new (source->display_text,
+        priv->combo = gtk_file_chooser_button_new (_("Select a folder"),
                                                    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 
+        g_object_set (priv->combo, "local-only", FALSE, NULL);
+        gtk_entry_set_activates_default (GTK_ENTRY (priv->entry),
+                                         gtk_printer_option_get_activates_default (source));
+
         label = gtk_label_new_with_mnemonic (_("_Name:"));
         gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), priv->entry);
@@ -840,7 +847,7 @@ update_widgets (GtkPrinterOptionWidget *widget)
   switch (source->type)
     {
     case GTK_PRINTER_OPTION_TYPE_BOOLEAN:
-      if (strcmp (source->value, "True") == 0)
+      if (g_ascii_strcasecmp (source->value, "True") == 0)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->check), TRUE);
       else
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->check), FALSE);
@@ -866,7 +873,7 @@ update_widgets (GtkPrinterOptionWidget *widget)
             text = g_filename_to_utf8 (basename, -1, NULL, NULL, NULL);
 
             if (text != NULL)
-              gtk_entry_set_text (GTK_ENTRY (priv->entry), basename);
+              gtk_entry_set_text (GTK_ENTRY (priv->entry), text);
             if (g_path_is_absolute (dirname))
               gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (priv->combo),
                                                    dirname);
@@ -911,6 +918,3 @@ gtk_printer_option_widget_get_value (GtkPrinterOptionWidget *widget)
   
   return "";
 }
-
-#define __GTK_PRINTER_OPTION_WIDGET_C__
-#include "gtkaliasdef.c"
