@@ -30,7 +30,7 @@
 #include <math.h>
 #include <stdlib.h>
 
-#include "gtkscale.h"
+#include "gtkscaleprivate.h"
 #include "gtkiconfactory.h"
 #include "gtkicontheme.h"
 #include "gtkmarshalers.h"
@@ -77,7 +77,6 @@
 				 *    unrelated code portions otherwise
 				 */
 
-
 typedef struct _GtkScaleMark GtkScaleMark;
 
 struct _GtkScalePrivate
@@ -96,7 +95,7 @@ struct _GtkScaleMark
 {
   gdouble          value;
   gchar           *markup;
-  GtkPositionType  position;
+  GtkPositionType  position; /* always GTK_POS_TOP or GTK_POS_BOTTOM */
 };
 
 enum {
@@ -443,7 +442,7 @@ gtk_scale_init (GtkScale *scale)
   priv->draw_value = TRUE;
   priv->value_pos = GTK_POS_TOP;
   priv->digits = 1;
-  _gtk_range_set_round_digits (range, priv->digits);
+  gtk_range_set_round_digits (range, priv->digits);
 
   gtk_scale_orientation_notify (range, NULL);
   g_signal_connect (scale, "notify::orientation",
@@ -510,8 +509,8 @@ gtk_scale_get_property (GObject      *object,
 /**
  * gtk_scale_new:
  * @orientation: the scale's orientation.
- * @adjustment: the #GtkAdjustment which sets the range of the scale, or
- *              %NULL to create a new adjustment.
+ * @adjustment: (allow-none): the #GtkAdjustment which sets the range
+ *              of the scale, or %NULL to create a new adjustment.
  *
  * Creates a new #GtkScale.
  *
@@ -613,7 +612,7 @@ gtk_scale_set_digits (GtkScale *scale,
     {
       priv->digits = digits;
       if (priv->draw_value)
-        _gtk_range_set_round_digits (range, digits);
+        gtk_range_set_round_digits (range, digits);
 
       _gtk_scale_clear_layout (scale);
       gtk_widget_queue_resize (GTK_WIDGET (scale));
@@ -662,9 +661,9 @@ gtk_scale_set_draw_value (GtkScale *scale,
     {
       priv->draw_value = draw_value;
       if (draw_value)
-        _gtk_range_set_round_digits (GTK_RANGE (scale), priv->digits);
+        gtk_range_set_round_digits (GTK_RANGE (scale), priv->digits);
       else
-        _gtk_range_set_round_digits (GTK_RANGE (scale), -1);
+        gtk_range_set_round_digits (GTK_RANGE (scale), -1);
 
       _gtk_scale_clear_layout (scale);
 
@@ -792,17 +791,17 @@ gtk_scale_get_range_border (GtkRange  *range,
                             NULL);
 
 
+      gtk_scale_get_mark_label_size (scale, GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
+
       if (gtk_orientable_get_orientation (GTK_ORIENTABLE (scale)) == GTK_ORIENTATION_HORIZONTAL)
         {
-          gtk_scale_get_mark_label_size (scale, GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
           if (n1 > 0)
             border->top += h1 + value_spacing + slider_width / 2;
           if (n2 > 0)
-            border->bottom += h2 + value_spacing + slider_width / 2; 
+            border->bottom += h2 + value_spacing + slider_width / 2;
         }
       else
         {
-          gtk_scale_get_mark_label_size (scale, GTK_POS_LEFT, &n1, &w1, &h1, &n2, &w2, &h2);
           if (n1 > 0)
             border->left += w1 + value_spacing + slider_width / 2;
           if (n2 > 0)
@@ -897,8 +896,8 @@ gtk_scale_get_mark_label_size (GtkScale        *scale,
           pango_layout_set_markup (layout, mark->markup, -1);
           pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
-	  w = logical_rect.width;
-	  h = logical_rect.height;
+          w = logical_rect.width;
+          h = logical_rect.height;
         }
       else
         {
@@ -991,7 +990,7 @@ gtk_scale_get_preferred_height (GtkWidget *widget,
 
       gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
 
-      gtk_scale_get_mark_label_size (GTK_SCALE (widget), GTK_POS_LEFT, &n1, &w1, &h1, &n2, &w2, &h2);
+      gtk_scale_get_mark_label_size (GTK_SCALE (widget), GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
       h1 = (n1 - 1) * h1 + MAX (h1, slider_length);
       h2 = (n2 - 1) * h1 + MAX (h2, slider_length);
       h = MAX (h1, h2);
@@ -1021,7 +1020,7 @@ find_next_pos (GtkWidget      *widget,
     }
 
   gtk_widget_get_allocation (widget, &allocation);
-  if (pos == GTK_POS_TOP || pos == GTK_POS_BOTTOM)
+  if (gtk_orientable_get_orientation (GTK_ORIENTABLE (widget)) == GTK_ORIENTATION_HORIZONTAL)
     return allocation.width;
   else
     return allocation.height;
@@ -1036,7 +1035,6 @@ gtk_scale_draw (GtkWidget *widget,
   GtkRange *range = GTK_RANGE (scale);
   GtkStateFlags state = 0;
   GtkStyleContext *context;
-  gint n_marks;
   gint *marks;
   gint focus_padding;
   gint slider_width;
@@ -1046,8 +1044,8 @@ gtk_scale_draw (GtkWidget *widget,
   context = gtk_widget_get_style_context (widget);
   gtk_widget_style_get (widget,
                         "focus-padding", &focus_padding,
-                        "slider-width", &slider_width, 
-                        "value-spacing", &value_spacing, 
+                        "slider-width", &slider_width,
+                        "value-spacing", &value_spacing,
                         NULL);
 
   /* We need to chain up _first_ so the various geometry members of
@@ -1071,7 +1069,7 @@ gtk_scale_draw (GtkWidget *widget,
       gint min_pos, max_pos;
 
       orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (range));
-      n_marks = _gtk_range_get_stop_positions (range, &marks);
+      _gtk_range_get_stop_positions (range, &marks);
       layout = gtk_widget_create_pango_layout (widget, NULL);
       gtk_range_get_range_rect (range, &range_rect);
 
@@ -1096,15 +1094,14 @@ gtk_scale_draw (GtkWidget *widget,
                   y1 = range_rect.y + range_rect.height;
                   y2 = y1 + slider_width / 2;
                   min_pos = min_pos_after;
-                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP, 0) - min_sep;
+                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_BOTTOM, 1) - min_sep;
                 }
 
               gtk_style_context_save (context);
               gtk_style_context_add_class (context, GTK_STYLE_CLASS_MARK);
               gtk_style_context_set_state (context, state);
 
-              gtk_render_line (context, cr,
-                               x1, y1, x1, y2);
+              gtk_render_line (context, cr, x1, y1, x1, y2);
 
               if (mark->markup)
                 {
@@ -1129,27 +1126,26 @@ gtk_scale_draw (GtkWidget *widget,
                       min_pos_after = x3 + logical_rect.width + min_sep;
                     }
 
-                  gtk_render_layout (context, cr,
-                                     x3, y3, layout);
+                  gtk_render_layout (context, cr, x3, y3, layout);
                 }
 
               gtk_style_context_restore (context);
             }
           else
             {
-              if (mark->position == GTK_POS_LEFT)
+              if (mark->position == GTK_POS_TOP)
                 {
                   x1 = range_rect.x;
                   x2 = range_rect.x - slider_width / 2;
                   min_pos = min_pos_before;
-                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_LEFT, 1) - min_sep;
+                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP, 1) - min_sep;
                 }
               else
                 {
                   x1 = range_rect.x + range_rect.width;
                   x2 = range_rect.x + range_rect.width + slider_width / 2;
                   min_pos = min_pos_after;
-                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_LEFT, 0) - min_sep;
+                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_BOTTOM, 1) - min_sep;
                 }
               y1 = marks[i];
 
@@ -1157,8 +1153,7 @@ gtk_scale_draw (GtkWidget *widget,
               gtk_style_context_add_class (context, GTK_STYLE_CLASS_MARK);
               gtk_style_context_set_state (context, state);
 
-              gtk_render_line (context, cr,
-                               x1, y1, x2, y1);
+              gtk_render_line (context, cr, x1, y1, x2, y1);
 
               if (mark->markup)
                 {
@@ -1172,7 +1167,7 @@ gtk_scale_draw (GtkWidget *widget,
                     y3 = max_pos - logical_rect.height;
                   if (y3 < 0)
                     y3 = 0;
-                  if (mark->position == GTK_POS_LEFT)
+                  if (mark->position == GTK_POS_TOP)
                     {
                       x3 = x2 - value_spacing - logical_rect.width;
                       min_pos_before = y3 + logical_rect.height + min_sep;
@@ -1183,13 +1178,12 @@ gtk_scale_draw (GtkWidget *widget,
                       min_pos_after = y3 + logical_rect.height + min_sep;
                     }
 
-                  gtk_render_layout (context, cr,
-                                     x3, y3, layout);
+                  gtk_render_layout (context, cr, x3, y3, layout);
                 }
 
               gtk_style_context_restore (context);
             }
-        } 
+        }
 
       g_object_unref (layout);
       g_free (marks);
@@ -1197,13 +1191,11 @@ gtk_scale_draw (GtkWidget *widget,
 
   if (priv->draw_value)
     {
-      GtkOrientation orientation;
       GtkAllocation allocation;
 
       PangoLayout *layout;
       gint x, y;
 
-      orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (range));
       layout = gtk_scale_get_layout (scale);
       gtk_scale_get_layout_offsets (scale, &x, &y);
       gtk_widget_get_allocation (widget, &allocation);
@@ -1400,8 +1392,8 @@ gtk_scale_get_layout (GtkScale *scale)
 /**
  * gtk_scale_get_layout_offsets:
  * @scale: a #GtkScale
- * @x: (allow-none): location to store X offset of layout, or %NULL
- * @y: (allow-none): location to store Y offset of layout, or %NULL
+ * @x: (out) (allow-none): location to store X offset of layout, or %NULL
+ * @y: (out) (allow-none): location to store Y offset of layout, or %NULL
  *
  * Obtains the coordinates where the scale will draw the 
  * #PangoLayout representing the text in the scale. Remember
@@ -1466,6 +1458,7 @@ void
 gtk_scale_clear_marks (GtkScale *scale)
 {
   GtkScalePrivate *priv;
+  GtkStyleContext *context;
 
   g_return_if_fail (GTK_IS_SCALE (scale));
 
@@ -1475,6 +1468,10 @@ gtk_scale_clear_marks (GtkScale *scale)
   g_slist_free (priv->marks);
   priv->marks = NULL;
 
+  context = gtk_widget_get_style_context (GTK_WIDGET (scale));
+  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
+  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+
   _gtk_range_set_stop_values (GTK_RANGE (scale), NULL, 0);
 
   gtk_widget_queue_resize (GTK_WIDGET (scale));
@@ -1493,22 +1490,22 @@ compare_marks (gpointer a, gpointer b)
 /**
  * gtk_scale_add_mark:
  * @scale: a #GtkScale
- * @value: the value at which the mark is placed, must be between 
+ * @value: the value at which the mark is placed, must be between
  *   the lower and upper limits of the scales' adjustment
  * @position: where to draw the mark. For a horizontal scale, #GTK_POS_TOP
- *   is drawn above the scale, anything else below. For a vertical scale,
- *   #GTK_POS_LEFT is drawn to the left of the scale, anything else to the
- *   right.
+ *   and %GTK_POS_LEFT are drawn above the scale, anything else below.
+ *   For a vertical scale, #GTK_POS_LEFT and %GTK_POS_TOP are drawn to
+ *   the left of the scale, anything else to the right.
  * @markup: (allow-none): Text to be shown at the mark, using <link linkend="PangoMarkupFormat">Pango markup</link>, or %NULL
  *
  *
- * Adds a mark at @value. 
+ * Adds a mark at @value.
  *
- * A mark is indicated visually by drawing a tick mark next to the scale, 
- * and GTK+ makes it easy for the user to position the scale exactly at the 
+ * A mark is indicated visually by drawing a tick mark next to the scale,
+ * and GTK+ makes it easy for the user to position the scale exactly at the
  * marks value.
  *
- * If @markup is not %NULL, text is shown next to the tick mark. 
+ * If @markup is not %NULL, text is shown next to the tick mark.
  *
  * To remove marks from a scale, use gtk_scale_clear_marks().
  *
@@ -1525,6 +1522,8 @@ gtk_scale_add_mark (GtkScale        *scale,
   GSList *m;
   gdouble *values;
   gint n, i;
+  GtkStyleContext *context;
+  int all_pos;
 
   g_return_if_fail (GTK_IS_SCALE (scale));
 
@@ -1533,23 +1532,49 @@ gtk_scale_add_mark (GtkScale        *scale,
   mark = g_new (GtkScaleMark, 1);
   mark->value = value;
   mark->markup = g_strdup (markup);
-  mark->position = position;
- 
+  if (position == GTK_POS_LEFT ||
+      position == GTK_POS_TOP)
+    mark->position = GTK_POS_TOP;
+  else
+    mark->position = GTK_POS_BOTTOM;
+
   priv->marks = g_slist_insert_sorted (priv->marks, mark,
                                        (GCompareFunc) compare_marks);
 
+#define MARKS_ABOVE 1
+#define MARKS_BELOW 2
+
+  all_pos = 0;
   n = g_slist_length (priv->marks);
   values = g_new (gdouble, n);
   for (m = priv->marks, i = 0; m; m = m->next, i++)
     {
       mark = m->data;
       values[i] = mark->value;
+      if (mark->position == GTK_POS_TOP)
+        all_pos |= MARKS_ABOVE;
+      else
+        all_pos |= MARKS_BELOW;
     }
-  
+
   _gtk_range_set_stop_values (GTK_RANGE (scale), values, n);
 
   g_free (values);
 
+  /* Set the style classes for the slider, so it could
+   * point to the right direction when marks are present
+   */
+  context = gtk_widget_get_style_context (GTK_WIDGET (scale));
+
+  if (all_pos & MARKS_ABOVE)
+    gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+  else
+    gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+  if (all_pos & MARKS_BELOW)
+    gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
+  else
+    gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
+
   gtk_widget_queue_resize (GTK_WIDGET (scale));
 }
 
@@ -1674,7 +1699,11 @@ marks_start_element (GMarkupParseContext *context,
 
       mark = g_slice_new (MarkData);
       mark->value = value;
-      mark->position = position;
+      if (position == GTK_POS_LEFT ||
+          position == GTK_POS_TOP)
+        mark->position = GTK_POS_TOP;
+      else
+        mark->position = GTK_POS_BOTTOM;
       mark->markup = g_string_new ("");
       mark->context = g_strdup (msg_context);
       mark->translatable = translatable;
