@@ -49,6 +49,7 @@
 #include "gtkwidget.h"
 #include "gtkmarshalers.h"
 
+#include <math.h>
 
 #define DEFAULT_SLIDER_WIDTH    (36)
 #define DEFAULT_SLIDER_HEIGHT   (22)
@@ -162,17 +163,30 @@ gtk_switch_motion (GtkWidget      *widget,
       GtkStyleContext *context;
       GtkStateFlags state;
       GtkBorder padding;
+      gint width, focus_width, focus_pad;
+
+      gtk_widget_style_get (widget,
+                            "focus-line-width", &focus_width,
+                            "focus-padding", &focus_pad,
+                            NULL);
 
       context = gtk_widget_get_style_context (widget);
       state = gtk_widget_get_state_flags (widget);
+
+      gtk_style_context_save (context);
+      gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
       gtk_style_context_get_padding (context, state, &padding);
+      gtk_style_context_restore (context);
+      
       gtk_widget_get_allocation (widget, &allocation);
 
+      width = allocation.width - 2 * (focus_width + focus_pad);
+
       /* constrain the handle within the trough width */
-      if (position > (allocation.width / 2 - padding.right))
-        priv->handle_x = allocation.width / 2 - padding.right;
+      if (position > (width / 2) - padding.right)
+        priv->handle_x = width / 2 - padding.right;
       else if (position < padding.left)
-        priv->handle_x = padding.left;
+        priv->handle_x = 0;
       else
         priv->handle_x = position;
 
@@ -524,12 +538,15 @@ gtk_switch_draw (GtkWidget *widget,
   GtkStyleContext *context;
   GdkRectangle handle;
   PangoLayout *layout;
+  PangoFontDescription *desc;
+  const PangoFontDescription *style_desc;
   PangoRectangle rect;
   gint label_x, label_y;
   GtkStateFlags state;
   GtkBorder padding;
   gint focus_width, focus_pad;
   gint x, y, width, height;
+  gint font_size, style_font_size;
 
   gtk_widget_style_get (widget,
                         "focus-line-width", &focus_width,
@@ -586,6 +603,21 @@ gtk_switch_draw (GtkWidget *widget,
    * the state
    */
   layout = gtk_widget_create_pango_layout (widget, C_("switch", "ON"));
+
+  /* FIXME: this should be really done in the theme, but overriding font size
+   * from it doesn't currently work. So we have to hardcode this here and
+   * below for the "OFF" label.
+   */
+  desc = pango_font_description_new ();
+
+  style_desc = gtk_style_context_get_font (context, state);
+  style_font_size = pango_font_description_get_size (style_desc);
+  font_size = MAX (style_font_size - 1 * PANGO_SCALE, ceil (style_font_size * PANGO_SCALE_SMALL));
+
+  pango_font_description_set_size (desc, font_size);
+
+  pango_layout_set_font_description (layout, desc);
+
   pango_layout_get_extents (layout, NULL, &rect);
   pango_extents_to_pixels (&rect, NULL);
 
@@ -600,6 +632,8 @@ gtk_switch_draw (GtkWidget *widget,
    * glyphs then use WHITE CIRCLE (U+25CB) as the text for the state
    */
   layout = gtk_widget_create_pango_layout (widget, C_("switch", "OFF"));
+  pango_layout_set_font_description (layout, desc);
+
   pango_layout_get_extents (layout, NULL, &rect);
   pango_extents_to_pixels (&rect, NULL);
 
@@ -621,6 +655,8 @@ gtk_switch_draw (GtkWidget *widget,
 
   gtk_switch_paint_handle (widget, cr, &handle);
 
+  pango_font_description_free (desc);
+
   return FALSE;
 }
 
