@@ -329,7 +329,7 @@ _gtk_theming_engine_set_context (GtkThemingEngine *engine,
 }
 
 /**
- * gtk_theming_engine_register_property:
+ * gtk_theming_engine_register_property: (skip)
  * @name_space: namespace for the property name
  * @parse_func: parsing function to use, or %NULL
  * @pspec: the #GParamSpec for the new property
@@ -473,7 +473,7 @@ gtk_theming_engine_get (GtkThemingEngine *engine,
  * gtk_theming_engine_get_style_property:
  * @engine: a #GtkThemingEngine
  * @property_name: the name of the widget style property
- * @value: (out) (transfer full): Return location for the property value, free with
+ * @value: Return location for the property value, free with
  *         g_value_unset() after use.
  *
  * Gets the value for a widget style property.
@@ -881,8 +881,9 @@ gtk_theming_engine_get_margin (GtkThemingEngine *engine,
  *
  * Returns the font description for a given state.
  *
- * Returns: the #PangoFontDescription for the given state. This
- *          object is owned by GTK+ and should not be freed.
+ * Returns: (transfer none): the #PangoFontDescription for the given
+ *          state. This object is owned by GTK+ and should not be
+ *          freed.
  *
  * Since: 3.0
  **/
@@ -1029,7 +1030,7 @@ gtk_theming_engine_load (const gchar *name)
  *
  * Returns the #GdkScreen to which @engine currently rendering to.
  *
- * Returns: a #GdkScreen, or %NULL.
+ * Returns: (transfer none): a #GdkScreen, or %NULL.
  **/
 GdkScreen *
 gtk_theming_engine_get_screen (GtkThemingEngine *engine)
@@ -1183,10 +1184,8 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
   gint exterior_size, interior_size, pad, thickness, border_width;
   GtkBorderStyle border_style;
   GtkBorder *border;
-  gdouble radius;
 
   flags = gtk_theming_engine_get_state (engine);
-  radius = MIN (width, height) / 2 - 0.5;
 
   cairo_save (cr);
 
@@ -1214,10 +1213,10 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
 
       cairo_new_sub_path (cr);
       cairo_arc (cr,
-		 x + exterior_size / 2.,
-		 y + exterior_size / 2.,
-		 (exterior_size - 1) / 2.,
-		 0, 2 * G_PI);
+                 x + exterior_size / 2.,
+                 y + exterior_size / 2.,
+                 (exterior_size - 1) / 2.,
+                 0, 2 * G_PI);
 
       gdk_cairo_set_source_rgba (cr, bg_color);
       cairo_fill_preserve (cr);
@@ -1251,10 +1250,10 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
       line_thickness = MAX (1, (3 + interior_size * 2) / 7);
 
       cairo_rectangle (cr,
-		       x + pad,
-		       y + pad + (interior_size - line_thickness) / 2.,
-		       interior_size,
-		       line_thickness);
+                       x + pad,
+                       y + pad + (interior_size - line_thickness) / 2.,
+                       interior_size,
+                       line_thickness);
       cairo_fill (cr);
     }
   if (flags & GTK_STATE_FLAG_ACTIVE)
@@ -1263,17 +1262,17 @@ gtk_theming_engine_render_option (GtkThemingEngine *engine,
       interior_size = MAX (1, exterior_size - 2 * pad);
 
       if (interior_size < 5)
-	{
-	  interior_size = 7;
-	  pad = MAX (0, (exterior_size - interior_size) / 2);
-	}
+        {
+          interior_size = 7;
+          pad = MAX (0, (exterior_size - interior_size) / 2);
+        }
 
       cairo_new_sub_path (cr);
       cairo_arc (cr,
-		 x + pad + interior_size / 2.,
-		 y + pad + interior_size / 2.,
-		 interior_size / 2.,
-		 0, 2 * G_PI);
+                 x + pad + interior_size / 2.,
+                 y + pad + interior_size / 2.,
+                 interior_size / 2.,
+                 0, 2 * G_PI);
       cairo_fill (cr);
     }
 
@@ -1473,6 +1472,32 @@ _cairo_round_rectangle_sides (cairo_t          *cr,
     }
 }
 
+/* Set the appropriate matrix for
+ * patterns coming from the style context
+ */
+static void
+style_pattern_set_matrix (cairo_pattern_t *pattern,
+                          gdouble          width,
+                          gdouble          height)
+{
+  cairo_matrix_t matrix;
+  gint w, h;
+
+  if (cairo_pattern_get_type (pattern) == CAIRO_PATTERN_TYPE_SURFACE)
+    {
+      cairo_surface_t *surface;
+
+      cairo_pattern_get_surface (pattern, &surface);
+      w = cairo_image_surface_get_width (surface);
+      h = cairo_image_surface_get_height (surface);
+    }
+  else
+    w = h = 1;
+
+  cairo_matrix_init_scale (&matrix, (gdouble) w / width, (gdouble) h / height);
+  cairo_pattern_set_matrix (pattern, &matrix);
+}
+
 static void
 render_background_internal (GtkThemingEngine *engine,
                             cairo_t          *cr,
@@ -1487,22 +1512,56 @@ render_background_internal (GtkThemingEngine *engine,
   GtkStateFlags flags;
   gboolean running;
   gdouble progress, alpha = 1;
-  gint radius;
+  GtkBorder *border;
+  gint radius, border_width;
+  GtkBorderStyle border_style;
+  gdouble mat_w, mat_h;
+  cairo_matrix_t identity;
+
+  /* Use unmodified size for pattern scaling */
+  mat_w = width;
+  mat_h = height;
 
   flags = gtk_theming_engine_get_state (engine);
-  cairo_save (cr);
+
+  cairo_matrix_init_identity (&identity);
 
   gtk_theming_engine_get (engine, flags,
                           "background-image", &pattern,
                           "background-color", &bg_color,
                           "border-radius", &radius,
+                          "border-width", &border,
+                          "border-style", &border_style,
                           NULL);
 
+  border_width = MIN (MIN (border->top, border->bottom),
+                      MIN (border->left, border->right));
+
+  if (border_width > 1 &&
+      border_style == GTK_BORDER_STYLE_NONE)
+    {
+      cairo_set_line_width (cr, border_width);
+
+      x += (gdouble) border_width / 2;
+      y += (gdouble) border_width / 2;
+      width -= border_width;
+      height -= border_width;
+    }
+  else
+    {
+      x += border->left;
+      y += border->top;
+      width -= border->left + border->right;
+      height -= border->top + border->bottom;
+    }
+
+  if (width <= 0 || height <= 0)
+    return;
+
+  cairo_save (cr);
+  cairo_translate (cr, x, y);
+
   running = gtk_theming_engine_state_is_running (engine, GTK_STATE_PRELIGHT, &progress);
-  _cairo_round_rectangle_sides (cr, (gdouble) radius,
-                                x, y, width, height,
-                                SIDE_ALL, junction);
-  cairo_clip (cr);
 
   if (gtk_theming_engine_has_class (engine, "background"))
     {
@@ -1511,9 +1570,6 @@ render_background_internal (GtkThemingEngine *engine,
       cairo_paint (cr);
     }
 
-  cairo_translate (cr, x, y);
-  cairo_scale (cr, width, height);
-
   if (running)
     {
       cairo_pattern_t *other_pattern;
@@ -1604,10 +1660,16 @@ render_background_internal (GtkThemingEngine *engine,
               /* Different pattern types, or different color
                * stop counts, alpha blend both patterns.
                */
-              cairo_rectangle (cr, 0, 0, 1, 1);
+              _cairo_round_rectangle_sides (cr, (gdouble) radius,
+                                            0, 0, width, height,
+                                            SIDE_ALL, junction);
+
+              style_pattern_set_matrix (other_pattern, mat_w, mat_h);
               cairo_set_source (cr, other_pattern);
               cairo_fill_preserve (cr);
 
+              cairo_pattern_set_matrix (other_pattern, &identity);
+
               /* Set alpha for posterior drawing
                * of the target pattern
                */
@@ -1691,30 +1753,57 @@ render_background_internal (GtkThemingEngine *engine,
         gdk_rgba_free (other_bg);
     }
 
-  cairo_rectangle (cr, 0, 0, 1, 1);
-
+  _cairo_round_rectangle_sides (cr, (gdouble) radius,
+                                0, 0, width, height,
+                                SIDE_ALL, junction);
   if (pattern)
     {
+      style_pattern_set_matrix (pattern, mat_w, mat_h);
       cairo_set_source (cr, pattern);
-      cairo_pattern_destroy (pattern);
     }
   else
     gdk_cairo_set_source_rgba (cr, bg_color);
 
   if (alpha == 1)
-    cairo_fill (cr);
+    {
+      if (border_width > 1 &&
+          border_style != GTK_BORDER_STYLE_NONE)
+        {
+          /* stroke with the same source, so the background
+           * has exactly the shape than the frame, this
+           * is important so gtk_render_background() and
+           * gtk_render_frame() fit perfectly with round
+           * borders.
+           */
+          cairo_fill_preserve (cr);
+          cairo_stroke (cr);
+        }
+      else
+        cairo_fill (cr);
+    }
   else
     {
-      cairo_pattern_t *mask;
+      cairo_save (cr);
+      _cairo_round_rectangle_sides (cr, (gdouble) radius,
+                                    0, 0, width, height,
+                                    SIDE_ALL, junction);
+      cairo_clip (cr);
 
-      mask = cairo_pattern_create_rgba (1, 1, 1, alpha);
-      cairo_mask (cr, mask);
-      cairo_pattern_destroy (mask);
+      cairo_paint_with_alpha (cr, alpha);
+
+      cairo_restore (cr);
+    }
+
+  if (pattern)
+    {
+      cairo_pattern_set_matrix (pattern, &identity);
+      cairo_pattern_destroy (pattern);
     }
 
   cairo_restore (cr);
 
   gdk_rgba_free (bg_color);
+  gtk_border_free (border);
 }
 
 static void
@@ -1726,35 +1815,12 @@ gtk_theming_engine_render_background (GtkThemingEngine *engine,
                                       gdouble           height)
 {
   GtkJunctionSides junction;
-  GtkStateFlags flags;
-  GtkBorder *border;
 
   junction = gtk_theming_engine_get_junction_sides (engine);
 
-  if (gtk_theming_engine_has_class (engine, "spinbutton") &&
-      gtk_theming_engine_has_class (engine, "button"))
-    {
-      x += 2;
-      y += 2;
-      width -= 4;
-      height -= 4;
-    }
-
-  flags = gtk_theming_engine_get_state (engine);
-  gtk_theming_engine_get (engine, flags,
-                          "border-width", &border,
-                          NULL);
-
-  x += border->left;
-  y += border->top;
-  width -= border->left + border->right;
-  height -= border->top + border->bottom;
-
   render_background_internal (engine, cr,
                               x, y, width, height,
                               junction);
-
-  gtk_border_free (border);
 }
 
 /* Renders the small triangle on corners so
@@ -2269,7 +2335,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
   const PangoMatrix *matrix;
   GdkRGBA *fg_color;
   GtkStateFlags flags;
-  GdkScreen *screen;
   gdouble progress;
   gboolean running;
 
@@ -2280,7 +2345,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
                           "color", &fg_color,
                           NULL);
 
-  screen = gtk_theming_engine_get_screen (engine);
   matrix = pango_context_get_matrix (pango_layout_get_context (layout));
 
   running = gtk_theming_engine_state_is_running (engine, GTK_STATE_PRELIGHT, &progress);
@@ -2317,7 +2381,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
   if (matrix)
     {
       cairo_matrix_t cairo_matrix;
-      PangoMatrix tmp_matrix;
       PangoRectangle rect;
 
       cairo_matrix_init (&cairo_matrix,
@@ -2329,7 +2392,6 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
       pango_matrix_transform_rectangle (matrix, &rect);
       pango_extents_to_pixels (&rect, NULL);
 
-      tmp_matrix = *matrix;
       cairo_matrix.x0 += x - rect.x;
       cairo_matrix.y0 += y - rect.y;
 
@@ -2340,8 +2402,14 @@ gtk_theming_engine_render_layout (GtkThemingEngine *engine,
 
   if (flags & GTK_STATE_FLAG_INSENSITIVE)
     {
+      GdkRGBA bg;
+
+      gtk_theming_engine_get_background_color (engine, flags, &bg);
+
       cairo_save (cr);
-      cairo_set_source_rgb (cr, 1, 1, 1);
+
+      gdk_cairo_set_source_rgba (cr, &bg);
+
       cairo_move_to (cr, x + 1, y + 1);
       _gtk_pango_fill_layout (cr, layout);
       cairo_restore (cr);
@@ -2600,12 +2668,14 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
   GtkStateFlags flags;
   GdkRGBA *bg_color;
   GdkRGBA lighter, darker;
+  GtkJunctionSides sides;
   gint xx, yy;
 
   cairo_save (cr);
   flags = gtk_theming_engine_get_state (engine);
 
-  cairo_set_line_width (cr, 1);
+  cairo_set_line_width (cr, 1.0);
+  sides = gtk_theming_engine_get_junction_sides (engine);
 
   gtk_theming_engine_get (engine, flags,
                           "background-color", &bg_color,
@@ -2613,20 +2683,10 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
   color_shade (bg_color, 0.7, &darker);
   color_shade (bg_color, 1.3, &lighter);
 
-  gdk_cairo_set_source_rgba (cr, bg_color);
-  cairo_rectangle (cr, x, y, width, height);
-  cairo_fill (cr);
+  render_background_internal (engine, cr, x, y, width, height, sides);
 
   if (gtk_theming_engine_has_class (engine, GTK_STYLE_CLASS_GRIP))
     {
-      GtkJunctionSides sides;
-      gint skip = -1;
-
-      cairo_save (cr);
-
-      cairo_set_line_width (cr, 1.0);
-      sides = gtk_theming_engine_get_junction_sides (engine);
-
       /* reduce confusing values to a meaningful state */
       if ((sides & (GTK_JUNCTION_CORNER_TOPLEFT | GTK_JUNCTION_CORNER_BOTTOMRIGHT)) == (GTK_JUNCTION_CORNER_TOPLEFT | GTK_JUNCTION_CORNER_BOTTOMRIGHT))
         sides &= ~GTK_JUNCTION_CORNER_TOPLEFT;
@@ -2649,8 +2709,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
             height = width;
           else if (height < width)
             width = height;
-
-          skip = 2;
         }
       else if (sides == GTK_JUNCTION_CORNER_BOTTOMLEFT)
         {
@@ -2662,8 +2720,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
             }
           else if (height < width)
             width = height;
-
-          skip = 1;
         }
       else if (sides == GTK_JUNCTION_RIGHT)
         {
@@ -2683,8 +2739,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               x += (width - height);
               width = height;
             }
-
-          skip = 3;
         }
       else if (sides == GTK_JUNCTION_CORNER_BOTTOMRIGHT)
         {
@@ -2699,8 +2753,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               x += (width - height);
               width = height;
             }
-
-          skip = 0;
         }
       else if (sides == GTK_JUNCTION_TOP)
         {
@@ -2756,7 +2808,7 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               gdk_cairo_set_source_rgba (cr, &darker);
               add_path_line (cr, x, yi, x + width, yi);
               cairo_stroke (cr);
-              yi+= 2;
+              yi += 2;
             }
         }
       else if (sides == GTK_JUNCTION_CORNER_TOPLEFT)
@@ -2879,8 +2931,6 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
               yi += 3;
             }
         }
-
-      cairo_restore (cr);
     }
   else if (gtk_theming_engine_has_class (engine, GTK_STYLE_CLASS_PANE_SEPARATOR))
     {
@@ -2894,11 +2944,11 @@ gtk_theming_engine_render_handle (GtkThemingEngine *engine,
   else
     {
       for (yy = y; yy < y + height; yy += 3)
-	for (xx = x; xx < x + width; xx += 6)
-	  {
-	    render_dot (cr, &lighter, &darker, xx, yy, 2);
-	    render_dot (cr, &lighter, &darker, xx + 3, yy + 1, 2);
-	  }
+        for (xx = x; xx < x + width; xx += 6)
+          {
+            render_dot (cr, &lighter, &darker, xx, yy, 2);
+            render_dot (cr, &lighter, &darker, xx + 3, yy + 1, 2);
+          }
     }
 
   cairo_restore (cr);
@@ -2919,7 +2969,6 @@ gtk_theming_engine_render_activity (GtkThemingEngine *engine,
       GtkStateFlags state;
       guint num_steps, step;
       GdkRGBA *color;
-      gdouble dx, dy;
       gdouble progress;
       gdouble radius;
       gdouble half;
@@ -2939,13 +2988,10 @@ gtk_theming_engine_render_activity (GtkThemingEngine *engine,
 
       cairo_save (cr);
 
-      cairo_translate (cr, x, y);
-
-      /* draw clip region */
       cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+      cairo_set_line_width (cr, 2.0);
+      cairo_translate (cr, x + width / 2, y + height / 2);
 
-      dx = width / 2;
-      dy = height / 2;
       radius = MIN (width / 2, height / 2);
       half = num_steps / 2;
 
@@ -2954,10 +3000,9 @@ gtk_theming_engine_render_activity (GtkThemingEngine *engine,
           gint inset = 0.7 * radius;
 
           /* transparency is a function of time and intial value */
-          gdouble t = (gdouble) ((i + num_steps - step)
-                                 % num_steps) / num_steps;
-
-          cairo_save (cr);
+          gdouble t = 1.0 - (gdouble) ((i + step) % num_steps) / num_steps;
+          gdouble xscale = - sin (i * G_PI / half);
+          gdouble yscale = - cos (i * G_PI / half);
 
           cairo_set_source_rgba (cr,
                                  color->red,
@@ -2965,16 +3010,13 @@ gtk_theming_engine_render_activity (GtkThemingEngine *engine,
                                  color->blue,
                                  color->alpha * t);
 
-          cairo_set_line_width (cr, 2.0);
           cairo_move_to (cr,
-                         dx + (radius - inset) * cos (i * G_PI / half),
-                         dy + (radius - inset) * sin (i * G_PI / half));
+                         (radius - inset) * xscale,
+                         (radius - inset) * yscale);
           cairo_line_to (cr,
-                         dx + radius * cos (i * G_PI / half),
-                         dy + radius * sin (i * G_PI / half));
+                         radius * xscale,
+                         radius * yscale);
           cairo_stroke (cr);
-
-          cairo_restore (cr);
         }
 
       cairo_restore (cr);
@@ -3017,6 +3059,55 @@ lookup_icon_size (GtkThemingEngine *engine,
   return gtk_icon_size_lookup_for_settings (settings, size, width, height);
 }
 
+/* Kudos to the gnome-panel guys. */
+static void
+colorshift_pixbuf (GdkPixbuf *src,
+                   GdkPixbuf *dest,
+                   gint       shift)
+{
+  gint i, j;
+  gint width, height, has_alpha, src_rowstride, dest_rowstride;
+  guchar *target_pixels;
+  guchar *original_pixels;
+  guchar *pix_src;
+  guchar *pix_dest;
+  int val;
+  guchar r, g, b;
+
+  has_alpha       = gdk_pixbuf_get_has_alpha (src);
+  width           = gdk_pixbuf_get_width (src);
+  height          = gdk_pixbuf_get_height (src);
+  src_rowstride   = gdk_pixbuf_get_rowstride (src);
+  dest_rowstride  = gdk_pixbuf_get_rowstride (dest);
+  original_pixels = gdk_pixbuf_get_pixels (src);
+  target_pixels   = gdk_pixbuf_get_pixels (dest);
+
+  for (i = 0; i < height; i++)
+    {
+      pix_dest = target_pixels   + i * dest_rowstride;
+      pix_src  = original_pixels + i * src_rowstride;
+
+      for (j = 0; j < width; j++)
+        {
+          r = *(pix_src++);
+          g = *(pix_src++);
+          b = *(pix_src++);
+
+          val = r + shift;
+          *(pix_dest++) = CLAMP (val, 0, 255);
+
+          val = g + shift;
+          *(pix_dest++) = CLAMP (val, 0, 255);
+
+          val = b + shift;
+          *(pix_dest++) = CLAMP (val, 0, 255);
+
+          if (has_alpha)
+            *(pix_dest++) = *(pix_src++);
+        }
+    }
+}
+
 static GdkPixbuf *
 gtk_theming_engine_render_icon_pixbuf (GtkThemingEngine    *engine,
                                        const GtkIconSource *source,
@@ -3063,8 +3154,7 @@ gtk_theming_engine_render_icon_pixbuf (GtkThemingEngine    *engine,
       else if (state & GTK_STATE_FLAG_PRELIGHT)
         {
           stated = gdk_pixbuf_copy (scaled);
-          gdk_pixbuf_saturate_and_pixelate (scaled, stated,
-                                            1.2, FALSE);
+          colorshift_pixbuf (scaled, stated, 30);
           g_object_unref (scaled);
         }
       else
